//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCQMMMeasureFrequency.c   v5.9
// Desc: Sample program that measures frequency
// KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

#ifdef _WIN32_WCE

#include <string.h>
#include <Winsock2.h>

static int kbhit()
{
	int i;
	int result=0;

	
	result |= GetAsyncKeyState(VK_RETURN);
	result |= GetAsyncKeyState(VK_SPACE);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//number keys, 0-9
	for (i=48; i<=57; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//capital character keys, A-Z
	for (i=65; i<=90; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//lower case keys, a-z
	for (i=97; i<=122; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	return result;
}

#endif


// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main QMMDSCQMMMeasureFrequency
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif

BYTE result;    // returned error code
DSCCB dsccb;    // structure containing board settings
DSCB dscb;      // handle used to refer to the board
DSCQMM_MMR mmr; // master mode register, structure for programming the master mode
float freq;     // frequency measured
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int

int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//	  Iinitialize the driver, using the driver version for validation
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the QMM board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nQMM BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf("%hx", &dsccb.base_address );

	printf("Enter interrupt level (2-7) : ");
	scanf("%d", &intBuff);
	dsccb.int_level = (BYTE) intBuff;
	
	getchar();

	if(dscInitBoard(DSC_QMM, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. Master Mode Register settings
	//
	//	    Initialize the Quartz Master Mode register for counter group 1.
	//		For this demo we can set all the values to default.
	//
	//=========================================================================

	memset(&mmr, 0, sizeof(DSCQMM_MMR));

	mmr.counter_group = 1;
	mmr.compare1_enable = 0;
	mmr.compare2_enable = 0;
	mmr.fout_divider = 8;
	mmr.fout_source = QMM_SOURCE_F3_40KHZ;
	mmr.tod_mode = 0;
		
	if( dscQMMSetMMR(dscb, &mmr) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscQMMSetMMR error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		getchar();
		return 0;
	}

	
	//=========================================================================
	// IV. Measure Frequency
	//
	//	    Call the Measure Frequency function after prompting user
	//		to wire pins appropriately.
	//
	//=========================================================================
	
	printf("Please wire counter 4\'s output to counter 5\'s gate (pin 15 to pin 12.)\n");
	printf("Attach the frequency to be measured to source 1 (pin 1).\n");

	printf("Press enter and the demo will repeatedly measure the frequency.\nThen press any key to exit.\n\n");

	while (!kbhit());

	getchar();

	while (!kbhit())
	{

		if ( dscQMMMeasureFrequency(dscb, QMM_INTERVAL_1S_1HZ, QMM_SOURCE_SRC1, &freq) != DE_NONE )
			printf("Frequency: %f OVERFLOW \n",freq);
		else
			printf("Frequency: %f \n",freq);

		dscSleep(500);
	}
	
	dscFree();
	return 0;
}


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	printf("For users running cycled mode: to EXIT, hold down the 'q' key\n");
	printf("on the keyboard then press 'Enter' when 'q' letters appear on screen\n\n");
	main();

	return 0;
}

#endif
