//=============================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
//	DSCQMMPulseModulation.c	v5.9
//	Sample Program that performs Pulse Width Modulation for the QMM
//	Created By KL
//=============================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main QMMDSCQMMPulseModulation
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


BYTE result;    // returned error code
DSCCB dsccb;    // structure containing board settings
DSCB dscb;      // handle used to refer to the board
DSCQMM_MMR mmr; // master mode register, structure for programming the master mode
DSCQMM_PWM pwm; // pulse width modulation, structure for programming the pwm
float dc;       // duty cycle of the pulse width modulation
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
float floatBuff;        // temp variable of size float
char ch;
int i;	   // miscellaneous counter
int quit;  // to stop the loop

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. MMR Settings
//	    IV. PWM Settings
//		 V. Perform Pulse Width Modulation
//	    VI. Cleanup
//
//=============================================================================

int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//	  Initialize the driver, using the driver version for validation
	//=========================================================================
	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DMM-16-AT board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nQMM BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf("%hx", &dsccb.base_address );

	dsccb.int_level = 3;

	if(dscInitBoard(DSC_QMM, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//======================================================================
	// III. MMR SETTINGS
	//
	//		Initialize the MMR settings
	//
	//======================================================================

	// These are default values.  The comparators and FOUT are not used in this
	// demo.

	memset(&mmr, 0, sizeof(DSCQMM_MMR));

	mmr.counter_group = 1;
	mmr.compare1_enable = 0;
	mmr.compare2_enable = 0;
	mmr.fout_divider = 8;
	mmr.fout_source = QMM_SOURCE_F2_400KHZ;
	mmr.tod_mode = 0;

	if( dscQMMSetMMR(dscb, &mmr) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscQMMSetMMR error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//======================================================================
	// IV. PWM SETTINGS
	//
	//		Initialize the MMR settings
	//
	//======================================================================

	printf("Enter counter you would like to use (1-5): ");
	scanf("%d", &intBuff);
	pwm.counter = (BYTE) intBuff;
	
	pwm.duty_cycle = 50;
	pwm.init = 1;

	printf("Enter pwm output frequency : ");
	scanf("%f", &floatBuff);
	pwm.output_freq = (FLOAT) floatBuff;

	printf("Current Duty cycle is set to 50%%\n");

	getchar();
	//======================================================================
	// V. PERFORM PULSE WIDTH MODULATION
	//
	//		Perform the modulation.
	//
	//======================================================================

	dc = 50.0f; // Duty cycle = 50%
	quit = 0; // don't quit yet

	printf("Press 'q' to exit \n");

	while (!quit)
	{
		pwm.duty_cycle = dc;

		printf("\n\nPWM error code: %d\n",dscQMMPulseWidthModulation(dscb, &pwm));

		printf("Duty Cycle:     %f\n",dc);
		printf("input_freq:     %d\n",pwm.input_freq);
		printf("hold_reg:       %d\n",pwm.hold_reg);
		printf("load_reg:       %d\n",pwm.load_reg);
		printf("hit_extreme:    %d\n",pwm.hit_extreme);
		printf("\n");

		printf("u     - Increase Duty Cycle by 3%%: \nd     - Decrease Duty Cycle by 3%%\nU     - Increase Duty Cycle by 0.1%%\nD     - Decrease Duty Cycle by 0.1%%\n");
				
		ch = getchar();
		if (ch == 'u') dc += (float)3;       // UP
		if (ch == 'd') dc -= (float)3;       // DOWN
		if (ch == 'U') dc -= (float)0.1;     // LEFT
		if (ch == 'D') dc += (float)0.1;     // RIGHT
		if (ch == 'q') { quit = 1; break; }
	}

	//QMM master reset
	dscRegisterWrite(dscb, 1, 0xFF);

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nDSCQMMPulseModulation completed.\n" );

	return 0;
}


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
